/** 
 @name AlignLayers
 @version 2.0 (05 June 2009)
 @fileoverview
 <h4>Description</h4>
 <p>This script aligns the selected layers along the x, y or z axis.
 Here the term 'aligns' must be interpreted as 'puts next to each other'.
 The first selected layer serves as a reference layer and it does not move.
 You can align layers having different sizes.
 A constant offset between two adjacent layers can also be specified.</p>
 <h4>Usage</h4>
 <ol>
    <li> In After Effects CS3 or later, run the script 
    <li> Select some layers (footage, comp, solid or text layers)
    <li> Click on one or more checkbox control to allow alignment along the corresponding axis
    <li> Optionally, move the slider control or edit text input to add a constant offset between consecutive layers
 </ol>
 <h4>Note</h4>
 <p>When a text layer is selected, the script assumes its anchor point to be located at the center of the layer. 
 You can use <a href="http://www.nabscripts.com/downloads/scripts/RepositionAnchorPoint.zip">RepositionAnchorPoint</a> to quickly reposition anchor point to the layer center.</p>
 @author Charles Bordenave
 @location http://www.nabscripts.com/downloads/scripts/AlignLayers.zip
*/


/**
 This class represents the main class of the script; 
 it is used to create the user interface which allows to align the selected layers
 @class Main class of the script
*/
function AlignLayers()
{
    // Variable used to keep track of 'this' reference
    var alignLayers = this;
    
    // Create an instance of the utils class to use its functions
    var utils = new AlignLayersUtils();    
    
    // Script infos
    this.scriptMinSupportVersion = "8.0";
    this.scriptName = "AlignLayers.jsx";    
    this.scriptVersion = "2.0";
    this.scriptTitle = "Align Layers";
    this.scriptCopyright = "Copyright (c) 2009 Charles Bordenave";
    this.scriptHomepage = "http://www.nabscripts.com";
    this.scriptDescription = {en: "This script aligns the selected layers along the x, y, z axis, and can also add a constant offset between adjacent layers.\\r\\rHere the term \\'aligns\\' must be interpreted as \\'puts next to each other\\'.", fr:"Ce script aligne les calques sélectionnés et ajoute possiblement un espace constant entre chaque calque.\\r\\rIci le terme \\'aligne\\' doit être interprété comme \\'place l\\'un à côté de l\\'autre\\'."};
    this.scriptAbout = {en:this.scriptName + ", v" + this.scriptVersion + "\\r\\r" + this.scriptCopyright + "\\r" + this.scriptHomepage + "\\r\\r" + utils.loc(this.scriptDescription), fr:this.scriptName + ", v" + this.scriptVersion + "\\r\\r" + this.scriptCopyright + "\\r" + this.scriptHomepage + "\\r\\r" + utils.loc(this.scriptDescription)};        

    // Errors
    this.requirementErr = {en:"This script requires After Effects CS3 or later.", fr:"Ce script nécessite After Effects CS3 ou supérieur."};    
    this.noCompErr = {en:"Select at least two layers.", fr:"Sélectionnez au moins deux calques."};
    this.noLayersErr = {en:"Select at least two layers.", fr:"Sélectionnez au moins deux calques."};
    this.badSelLayersErr = {en:"The selected layers must be AVLayers or TextLayers.", fr:"Les calques séléctionnées doivent être des calques AudioVidéo ou des calques Texte."};
    
    // UI strings and default settings
    this.aboutBtnName = "?";
    this.xAlignCbName = {en:"X-alignment:", fr:"Alignement X:"};
    this.yAlignCbName = {en:"Y-alignment:", fr:"Alignement Y:"};
    this.zAlignCbName = {en:"Z-alignment:", fr:"Alignement Z:"};
    this.xAlignDflt = 0;
    this.yAlignDflt = 0;
    this.zAlignDflt = 0;

    // Internal data
    this.m_xAlign = false;
    this.m_yAlign = false;
    this.m_zAlign = false;
    this.m_xAlignVal = this.xAlignDflt;
    this.m_yAlignVal = this.yAlignDflt;
    this.m_zAlignVal = this.zAlignDflt;

    /**
     Creates and displays the script interface
     @param {Object} thisObj A Panel object if the script is launched from the Window menu, null otherwise    
    */
    this.buildUI = function (thisObj)
    {
        // dockable panel or palette
        var pal = (thisObj instanceof Panel) ? thisObj : new Window("palette", this.scriptTitle, undefined, {resizeable:true});
        
        // resource specifications
        var res =
        "group { orientation:'column', alignment:['fill','fill'], alignChildren:['right','top'], \
            gr1: Group { \
                aboutBtn: Button { text:'" + this.aboutBtnName + "', preferredSize:[25,20] } \
            }, \
            gr2: Group { orientation:'row', alignment:['fill','top'], \
                xAlignCb: Checkbox { text:'" + utils.loc(this.xAlignCbName) + "' }, \
                xAlignScl: Scrollbar { minvalue:-100, enabled:false, preferredSize:[150,20], alignment:['fill','center'] }, \
                xAlignEt: EditText { text:'" + this.xAlignDflt + "', characters:5, enabled:false, alignment:['right','center'] } \
            }, \
            gr3: Group { orientation:'row', alignment:['fill','top'], \
                yAlignCb: Checkbox { text:'" + utils.loc(this.yAlignCbName) + "' }, \
                yAlignScl: Scrollbar { minvalue:-100, enabled:false, preferredSize:[150,20], alignment:['fill','center'] }, \
                yAlignEt: EditText { text:'" + this.yAlignDflt + "', characters:5, enabled:false, alignment:['right','center'] } \
            }, \
              gr4: Group { orientation:'row', alignment:['fill','top'], \
                zAlignCb: Checkbox { text:'" + utils.loc(this.zAlignCbName) + "' }, \
                zAlignScl: Scrollbar { minvalue:-100, enabled:false, preferredSize:[150,20], alignment:['fill','center'] }, \
                zAlignEt: EditText { text:'" + this.zAlignDflt + "', characters:5, enabled:false, alignment:['right','center'] } \
            } \
        }"; 
        pal.gr = pal.add(res);
        
        pal.gr.gr2.xAlignEt.graphics.foregroundColor =
        pal.gr.gr3.yAlignEt.graphics.foregroundColor =
        pal.gr.gr4.zAlignEt.graphics.foregroundColor = pal.graphics.newPen(pal.graphics.BrushType.SOLID_COLOR, [0,0,0], 1);
        
        // event callbacks
        pal.onResizing = pal.onResize = function () 
        {
            this.layout.resize();
        };
                
        pal.gr.gr1.aboutBtn.onClick = function () 
        { 
            utils.createAboutDlg(alignLayers.scriptAbout); 
        };
        
        pal.gr.gr2.xAlignCb.onClick = function () 
        {
            if (this.value)
            {
                var err = alignLayers.positionSelectedLayers();
                if (err)
                {
                    utils.throwErr(err);
                    this.value = false;
                    this.parent.xAlignScl.enabled = false;
                    this.parent.xAlignEt.enabled = false;
                }
            }
            this.parent.xAlignScl.enabled = this.value;
            this.parent.xAlignEt.enabled = this.value; 
            alignLayers.m_xAlign = this.value;
        };
        
        pal.gr.gr2.xAlignScl.onChange = pal.gr.gr2.xAlignScl.onChanging = function () 
        {
            this.value = Math.round(this.value);
            
            alignLayers.m_xAlignVal = this.value;
            this.parent.xAlignEt.text = this.value;            
            
            var err = alignLayers.positionSelectedLayers();
            if (err)
            {
                utils.throwErr(err);
                this.parent.xAlignScl.enabled = false;
                this.parent.xAlignEt.enabled = false;
            }      
        };

        pal.gr.gr2.xAlignEt.onChange = function () 
        {
            if (isNaN(this.text))
            {
                this.text = alignLayers.xAlignDflt;
            }
            alignLayers.m_xAlignVal = Math.round(this.text);
            this.parent.xAlignScl.value = Math.round(this.text);
             
            var err = alignLayers.positionSelectedLayers();
            if (err)
            {
                utils.throwErr(err);
                this.parent.xAlignScl.enabled = false;
                this.parent.xAlignEt.enabled = false;
            }      
        };

        pal.gr.gr3.yAlignCb.onClick = function () 
        {
            if (this.value)
            {
                var err = alignLayers.positionSelectedLayers();
                if (err)
                {
                    utils.throwErr(err);
                    this.value = false;
                    this.parent.yAlignScl.enabled = false;
                    this.parent.yAlignEt.enabled = false;
                }
            }
            this.parent.yAlignScl.enabled = this.value;
            this.parent.yAlignEt.enabled = this.value; 
            alignLayers.m_yAlign = this.value;      
        };
        
        pal.gr.gr3.yAlignScl.onChange = pal.gr.gr3.yAlignScl.onChanging = function () 
        {
            this.value = Math.round(this.value);
            
            alignLayers.m_yAlignVal = this.value;
            this.parent.yAlignEt.text = this.value;            

            var err = alignLayers.positionSelectedLayers();
            if (err)
            {
                utils.throwErr(err);
                this.parent.yAlignScl.enabled = false;
                this.parent.yAlignEt.enabled = false;
            }
        };

        pal.gr.gr3.yAlignEt.onChange = function () 
        {
            if (isNaN(this.text))
            {
                this.text = alignLayers.yAlignDflt;
            }
            alignLayers.m_yAlignVal = Math.round(this.text);
            this.parent.yAlignScl.value = Math.round(this.text);
               
            var err = alignLayers.positionSelectedLayers();
            if (err)
            {
                utils.throwErr(err);
                this.parent.yAlignScl.enabled = false;
                this.parent.yAlignEt.enabled = false;
            }
        };
        
        pal.gr.gr4.zAlignCb.onClick = function () 
        {
            if (this.value)
            {
                var err = alignLayers.positionSelectedLayers();
                if (err)
                {
                    utils.throwErr(err);
                    this.value = false;
                    this.parent.zAlignScl.enabled = false;
                    this.parent.zAlignEt.enabled = false;
                }
            }
            this.parent.zAlignScl.enabled = this.value;
            this.parent.zAlignEt.enabled = this.value; 
            alignLayers.m_zAlign = this.value;      
        };
        
        pal.gr.gr4.zAlignScl.onChange = pal.gr.gr4.zAlignScl.onChanging = function () 
        {
            this.value = Math.round(this.value);
            
            alignLayers.m_zAlignVal = this.value;
            this.parent.zAlignEt.text = this.value;            

            var err = alignLayers.positionSelectedLayers();
            if (err)
            {
                utils.throwErr(err);
                this.parent.zAlignScl.enabled = false;
                this.parent.zAlignEt.enabled = false;
            }
        };

        pal.gr.gr4.zAlignEt.onChange = function () 
        {
            if (isNaN(this.text))
            {
                this.text = alignLayers.zAlignDflt;
            }
            alignLayers.m_zAlignVal = Math.round(this.text);
            this.parent.zAlignScl.value = Math.round(this.text);

            var err = alignLayers.positionSelectedLayers();
            if (err)
            {
                utils.throwErr(err);
                this.parent.zAlignScl.enabled = false;
                this.parent.zAlignEt.enabled = false;
            }            
        };        
            
        // show user interface
        if (pal instanceof Window)
        {
            pal.center();
            pal.show();
        }
        else
        {
            pal.layout.layout(true);
        }       
    };

    /**
     Determines whether the active item is a composition  
     @return True if the active item is not a composition, False otherwise
    */    
    this.checkActiveItem = function () 
    {
        var err = false;
        var comp = app.project.activeItem;
        if (!comp || !(comp instanceof CompItem))
        {
            err = true;
        }
        return err;
    };   

    /**
     Determines whether the layers are of correct type (footage, comp, solid or text layer)  
     @param {Array} layers An array of layers
     @return True if at least one layer has not the correct type, False otherwise    
    */    
    this.checkSelectedLayers = function (layers)
    {
        var err = false;        
        for (var i = 0; !err && i < layers.length; i++)
        {
            if (!(layers[i] instanceof AVLayer) && !(layers[i] instanceof TextLayer)) // reject camera/light layers
            {
                err = true;
            }
        }        
        return err;        
    };

    /**
     Functional part of the script: aligns selected layers along the x, y or z axis, and eventually adds a constant offset between each adjacent layer
    */    
    this.positionSelectedLayers = function ()
    {
        var comp = app.project.activeItem;
        var err = this.noCompErr;
        if (this.checkActiveItem(comp)) return err;

        var layers = comp.selectedLayers;
        var err = this.noLayersErr;
        if (layers.length < 2) return err;

        var err = this.badSelLayersErr;
        if (this.checkSelectedLayers(layers)) return err;

        var firstLayerPos = layers[0].position.valueAtTime(comp.time, true);
        var curPos = firstLayerPos;

        app.beginUndoGroup(this.scriptTitle);

        try
        {
            for (var i = 1; i < layers.length; i++) // skip the first selected layer
            {                
                var prevLayer = layers[i-1];
                var layer = layers[i];

                var xDir = this.m_xAlignVal < 0 ? -1 : 1;
                var yDir = this.m_yAlignVal < 0 ? -1 : 1; 

                var prevSx = prevLayer.scale.valueAtTime(comp.time, true)[0] / 100;
                var prevSy = prevLayer.scale.valueAtTime(comp.time, true)[1] / 100;

                var sx = layer.scale.valueAtTime(comp.time, true)[0] / 100;
                var sy = layer.scale.valueAtTime(comp.time, true)[1] / 100;

                var prevLayerSize = utils.getLayerSize(prevLayer, comp.time);
                var prevW = prevSx * prevLayerSize[0];
                var prevH = prevSy * prevLayerSize[1];

                var layerSize = utils.getLayerSize(layer, comp.time);
                var w = sx * layerSize[0];
                var h = sy * layerSize[1];

                var x = 0.5 * xDir * (prevW + w) + this.m_xAlignVal;
                var y = 0.5 * yDir * (prevH + h) + this.m_yAlignVal;
                var z = this.m_zAlignVal;

                if (layer.position.numKeys) // set new position keyframe at current comp time
                {
                    if (this.m_xAlign)
                    {
                        layer.position.setValueAtTime(comp.time, curPos + [x,0,0]);
                        curPos += [x,0,0];
                    }
                    if (this.m_yAlign)
                    {
                        layer.position.setValueAtTime(comp.time, curPos + [0,y,0]);
                        curPos += [0,y,0];
                    }
                    if (this.m_zAlign)
                    {
                        layer.threeDLayer = true;
                        layer.position.setValueAtTime(comp.time, curPos + [0,0,z]);
                        curPos += [0,0,z];
                    }               
                }
                else // set new position value
                {
                    if (this.m_xAlign)
                    {
                        layer.position.setValue(curPos + [x,0,0]);
                        curPos += [x,0,0];
                    }
                    if (this.m_yAlign)
                    {
                        layer.position.setValue(curPos + [0,y,0]);
                        curPos += [0,y,0];
                    }    
                    if (this.m_zAlign)
                    {
                        layer.threeDLayer = true;
                        layer.position.setValue(curPos + [0,0,z]);
                        curPos += [0,0,z];
                    }    
                }
            }
        }
        catch(e)
        { 
            var err = {en:e, fr:e}; 
            return err;
        }

        app.endUndoGroup();                        
    };
    
    /**
     Runs the script  
     @param {Object} thisObj A Panel object if the script is launched from the Window menu, null otherwise
    */
    this.run = function (thisObj) 
    {
        if (parseFloat(app.version) < parseFloat(this.scriptMinSupportVersion))
        {
            this.throwErr(this.requirementErr);
        }
        else
        {
            this.buildUI(thisObj);
        }    
    };
}


/**
 This class provides some utility functions used by AlignLayers
 @class Some utility functions grouped in a class
*/
function AlignLayersUtils()
{
    /**
     String localization function: english and french languages are supported
     @param {Object} str A localization object containing the localized versions of a string    
     @return Appropriate localized version of str
    */    
    this.loc = function (str)
    {
        return app.language == Language.FRENCH ? str.fr : str.en;
    };

    /**
     Displays a window containg a localized error message
     @param {Object} err A localization object containing the localized versions of an error message
    */    
    this.throwErr = function (err)
    {
        var wndTitle = $.fileName.substring($.fileName.lastIndexOf("/")+1, $.fileName.lastIndexOf("."));
        Window.alert("Script error:\r" + this.loc(err), wndTitle, true);
    };            

    /**
     Get the size in pixels of an AV layer (comp layer, footage layer, solid layer, text layer) at specific time
     @param {Object} avLayer An AV layer object     
     @param {Number} time A floating-point value representing the time in seconds at which the layer size must be evaluated    
     @return A two-dimensional array containing the width and height of the layer
    */
    this.getLayerSize = function (avLayer, time)
    {
        var w, h;
        if (!(avLayer instanceof TextLayer))
        {
            w = avLayer.width;
            h = avLayer.height;
        }
        else
        {
            var bb = avLayer.sourceRectAtTime(time, true);
            w = bb.width;
            h = bb.height;
        }
        return [w,h];
    }; 

    /**
     Displays a customized window containg the About text
     @param {String} aboutStr The text to display
    */
    this.createAboutDlg = function (aboutStr)
    {        
        eval(unescape('%20%20%20%20%20%20%20%20%2F%2A%2A%20%0A%20%20%20%20%20%20%20%20%20%44%72%61%77%20%73%6F%6D%65%20%72%61%6E%64%6F%6D%20%72%65%63%74%61%6E%67%6C%65%73%20%28%70%6F%73%69%74%69%6F%6E%2C%20%73%69%7A%65%2C%20%63%6F%6C%6F%72%2C%20%61%6C%70%68%61%29%20%6F%6E%20%74%68%65%20%77%69%6E%64%6F%77%20%62%61%63%6B%67%72%6F%75%6E%64%0A%20%20%20%20%20%20%20%20%20%40%69%67%6E%6F%72%65%20%0A%20%20%20%20%20%20%20%20%2A%2F%0A%20%20%20%20%20%20%20%20%66%75%6E%63%74%69%6F%6E%20%61%64%64%4E%61%62%73%63%72%69%70%74%73%42%61%63%6B%67%72%6F%75%6E%64%53%69%67%6E%61%74%75%72%65%28%77%6E%64%29%0A%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%6E%75%6D%52%65%63%74%20%3D%20%32%34%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%6D%69%6E%4F%70%61%63%69%74%79%20%3D%20%30%2E%30%35%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%6D%61%78%4F%70%61%63%69%74%79%20%3D%20%30%2E%31%35%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%6C%65%66%74%45%64%67%65%20%3D%20%30%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%74%6F%70%45%64%67%65%20%3D%20%30%3B%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%72%69%67%68%74%45%64%67%65%20%3D%20%77%6E%64%2E%77%69%6E%64%6F%77%42%6F%75%6E%64%73%2E%77%69%64%74%68%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%62%6F%74%74%6F%6D%45%64%67%65%20%3D%20%77%6E%64%2E%77%69%6E%64%6F%77%42%6F%75%6E%64%73%2E%68%65%69%67%68%74%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%66%6F%72%20%28%76%61%72%20%69%20%3D%20%30%20%3B%20%69%20%3C%20%6E%75%6D%52%65%63%74%3B%20%69%2B%2B%29%0A%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%78%4C%6F%63%20%3D%20%31%30%20%2B%20%28%72%69%67%68%74%45%64%67%65%20%2D%20%32%30%29%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%79%4C%6F%63%20%3D%20%31%30%20%2B%20%28%62%6F%74%74%6F%6D%45%64%67%65%20%2D%20%32%30%29%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%77%69%64%74%68%20%3D%20%35%20%2B%20%31%35%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%68%65%69%67%68%74%20%3D%20%35%20%2B%20%31%35%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%62%6F%72%64%65%72%57%69%64%74%68%20%3D%20%31%20%2B%20%34%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%62%6F%72%64%65%72%43%6F%6C%6F%72%20%3D%20%5B%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%2C%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%2C%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%2C%20%6D%69%6E%4F%70%61%63%69%74%79%20%2B%20%28%6D%61%78%4F%70%61%63%69%74%79%20%2D%20%6D%69%6E%4F%70%61%63%69%74%79%29%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%5D%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%63%6F%6C%6F%72%42%72%75%73%68%20%3D%20%77%6E%64%2E%67%72%61%70%68%69%63%73%2E%6E%65%77%42%72%75%73%68%28%77%6E%64%2E%67%72%61%70%68%69%63%73%2E%42%72%75%73%68%54%79%70%65%2E%53%4F%4C%49%44%5F%43%4F%4C%4F%52%2C%20%62%6F%72%64%65%72%43%6F%6C%6F%72%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%31%20%3D%20%77%6E%64%2E%61%64%64%28%22%67%72%6F%75%70%22%2C%20%5B%78%4C%6F%63%2C%20%79%4C%6F%63%2C%20%78%4C%6F%63%20%2B%20%77%69%64%74%68%2C%20%79%4C%6F%63%20%2B%20%62%6F%72%64%65%72%57%69%64%74%68%5D%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%32%20%3D%20%77%6E%64%2E%61%64%64%28%22%67%72%6F%75%70%22%2C%20%5B%78%4C%6F%63%2C%20%79%4C%6F%63%20%2B%20%68%65%69%67%68%74%20%2D%20%62%6F%72%64%65%72%57%69%64%74%68%2C%20%78%4C%6F%63%20%2B%20%77%69%64%74%68%2C%20%79%4C%6F%63%20%2B%20%68%65%69%67%68%74%5D%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%33%20%3D%20%77%6E%64%2E%61%64%64%28%22%67%72%6F%75%70%22%2C%20%5B%78%4C%6F%63%2C%20%79%4C%6F%63%20%2B%20%62%6F%72%64%65%72%57%69%64%74%68%2C%20%78%4C%6F%63%20%2B%20%62%6F%72%64%65%72%57%69%64%74%68%2C%20%79%4C%6F%63%20%2B%20%68%65%69%67%68%74%20%2D%20%62%6F%72%64%65%72%57%69%64%74%68%5D%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%34%20%3D%20%77%6E%64%2E%61%64%64%28%22%67%72%6F%75%70%22%2C%20%5B%78%4C%6F%63%20%2B%20%77%69%64%74%68%20%2D%20%62%6F%72%64%65%72%57%69%64%74%68%2C%20%79%4C%6F%63%20%2B%20%62%6F%72%64%65%72%57%69%64%74%68%2C%20%78%4C%6F%63%20%2B%20%77%69%64%74%68%2C%20%79%4C%6F%63%20%2B%20%68%65%69%67%68%74%20%2D%20%62%6F%72%64%65%72%57%69%64%74%68%5D%29%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%31%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%32%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%33%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%34%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%63%6F%6C%6F%72%42%72%75%73%68%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%7D%20%20%20%20%0A%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%76%61%72%20%64%6C%67%20%3D%20%6E%65%77%20%57%69%6E%64%6F%77%28%22%64%69%61%6C%6F%67%22%2C%20%22%41%62%6F%75%74%22%29%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%2F%2F%20%70%61%6E%65%6C%20%62%6F%72%64%65%72%53%74%79%6C%65%3A%20%6F%6E%65%20%6F%66%20%62%6C%61%63%6B%2C%20%65%74%63%68%65%64%2C%20%67%72%61%79%2C%20%72%61%69%73%65%64%2C%20%73%75%6E%6B%65%6E%2E%20%44%65%66%61%75%6C%74%20%69%73%20%65%74%63%68%65%64%2E%0A%20%20%20%20%20%20%20%20%2F%2F%20%72%65%73%6F%75%72%63%65%20%73%70%65%63%69%66%69%63%61%74%69%6F%6E%73%0A%20%20%20%20%20%20%20%20%76%61%72%20%72%65%73%20%3D%0A%20%20%20%20%20%20%20%20%22%67%72%6F%75%70%20%7B%20%6F%72%69%65%6E%74%61%74%69%6F%6E%3A%27%63%6F%6C%75%6D%6E%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%66%69%6C%6C%27%2C%27%66%69%6C%6C%27%5D%2C%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%61%62%6F%75%74%50%6E%6C%3A%20%50%61%6E%65%6C%20%7B%20%70%72%6F%70%65%72%74%69%65%73%3A%7B%20%62%6F%72%64%65%72%53%74%79%6C%65%3A%27%73%75%6E%6B%65%6E%27%20%7D%2C%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%61%62%6F%75%74%45%74%3A%20%45%64%69%74%54%65%78%74%20%7B%20%74%65%78%74%3A%27%22%20%2B%20%74%68%69%73%2E%6C%6F%63%28%61%62%6F%75%74%53%74%72%29%20%2B%20%22%27%2C%20%70%72%6F%70%65%72%74%69%65%73%3A%7B%6D%75%6C%74%69%6C%69%6E%65%3A%74%72%75%65%7D%2C%20%70%72%65%66%65%72%72%65%64%53%69%7A%65%3A%5B%32%38%30%2C%31%35%30%5D%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%72%69%67%68%74%27%2C%27%63%65%6E%74%65%72%27%5D%20%7D%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%7D%2C%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%62%74%6E%73%47%72%3A%20%47%72%6F%75%70%20%7B%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%66%69%6C%6C%27%2C%27%66%69%6C%6C%27%5D%2C%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%69%73%69%74%42%74%6E%3A%20%42%75%74%74%6F%6E%20%7B%20%74%65%78%74%3A%27%56%69%73%69%74%20%48%6F%6D%65%70%61%67%65%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%6C%65%66%74%27%2C%27%63%65%6E%74%65%72%27%5D%20%7D%2C%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%6F%6B%42%74%6E%3A%20%42%75%74%74%6F%6E%20%7B%20%74%65%78%74%3A%27%4F%6B%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%72%69%67%68%74%27%2C%27%63%65%6E%74%65%72%27%5D%20%7D%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%7D%20%5C%0A%20%20%20%20%20%20%20%20%7D%22%3B%20%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%20%3D%20%64%6C%67%2E%61%64%64%28%72%65%73%29%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%2F%2F%20%6F%6E%20%4D%61%63%20%77%65%20%63%61%6E%20%64%69%73%61%62%6C%65%20%65%64%69%74%20%74%65%78%74%20%77%68%69%6C%65%20%61%6C%6C%6F%77%69%6E%67%20%73%63%72%6F%6C%6C%69%6E%67%2C%20%6F%6E%20%57%69%6E%64%6F%77%73%20%77%65%20%63%61%6E%27%74%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%2E%61%62%6F%75%74%50%6E%6C%2E%61%62%6F%75%74%45%74%2E%65%6E%61%62%6C%65%64%20%3D%20%28%24%2E%6F%73%2E%69%6E%64%65%78%4F%66%28%22%57%69%6E%22%29%20%21%3D%20%2D%31%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%2F%2F%20%64%72%61%77%20%72%61%6E%64%6F%6D%20%62%61%63%6B%67%72%6F%75%6E%64%20%63%6F%6C%6F%72%20%28%67%72%61%79%73%63%61%6C%65%29%0A%20%20%20%20%20%20%20%20%69%66%20%28%70%61%72%73%65%46%6C%6F%61%74%28%61%70%70%2E%76%65%72%73%69%6F%6E%29%20%3E%3D%20%39%2E%30%29%20%2F%2F%20%43%53%34%20%6F%72%20%6C%61%74%65%72%0A%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%77%68%69%74%65%42%72%75%73%68%20%3D%20%64%6C%67%2E%67%72%61%70%68%69%63%73%2E%6E%65%77%42%72%75%73%68%28%64%6C%67%2E%67%72%61%70%68%69%63%73%2E%42%72%75%73%68%54%79%70%65%2E%53%4F%4C%49%44%5F%43%4F%4C%4F%52%2C%20%5B%31%2C%20%31%2C%20%31%2C%20%31%5D%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%72%61%6E%64%20%3D%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%62%67%42%72%75%73%68%20%3D%20%64%6C%67%2E%67%72%61%70%68%69%63%73%2E%6E%65%77%42%72%75%73%68%28%64%6C%67%2E%67%72%61%70%68%69%63%73%2E%42%72%75%73%68%54%79%70%65%2E%53%4F%4C%49%44%5F%43%4F%4C%4F%52%2C%20%5B%72%61%6E%64%2C%20%72%61%6E%64%2C%20%72%61%6E%64%2C%20%31%5D%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%62%67%42%72%75%73%68%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%2E%61%62%6F%75%74%50%6E%6C%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%77%68%69%74%65%42%72%75%73%68%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%64%6C%67%2E%6C%61%79%6F%75%74%2E%6C%61%79%6F%75%74%28%74%72%75%65%29%3B%20%2F%2F%20%74%6F%20%67%65%74%20%77%69%6E%64%6F%77%20%62%6F%75%6E%64%73%0A%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%61%64%64%4E%61%62%73%63%72%69%70%74%73%42%61%63%6B%67%72%6F%75%6E%64%53%69%67%6E%61%74%75%72%65%28%64%6C%67%29%3B%0A%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%2E%62%74%6E%73%47%72%2E%6F%6B%42%74%6E%2E%6F%6E%43%6C%69%63%6B%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%20%7B%20%64%6C%67%2E%63%6C%6F%73%65%28%29%3B%20%7D%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%2F%2F%20%6F%70%65%6E%20%68%6F%6D%65%70%61%67%65%20%75%72%6C%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%2E%62%74%6E%73%47%72%2E%76%69%73%69%74%42%74%6E%2E%6F%6E%43%6C%69%63%6B%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%20%0A%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%63%6D%64%20%3D%20%22%22%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%75%72%6C%20%3D%20%22%68%74%74%70%3A%2F%2F%77%77%77%2E%6E%61%62%73%63%72%69%70%74%73%2E%63%6F%6D%2F%22%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%69%66%20%28%24%2E%6F%73%2E%69%6E%64%65%78%4F%66%28%22%57%69%6E%22%29%20%21%3D%20%2D%31%29%0A%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%69%66%20%28%46%69%6C%65%28%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%22%29%2E%65%78%69%73%74%73%29%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%09%09%09%09%09%63%6D%64%20%2B%3D%20%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%20%22%20%2B%20%75%72%6C%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%65%6C%73%65%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%09%63%6D%64%20%2B%3D%20%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%49%6E%74%65%72%6E%65%74%20%45%78%70%6C%6F%72%65%72%2F%69%65%78%70%6C%6F%72%65%2E%65%78%65%20%22%20%2B%20%75%72%6C%3B%0A%09%09%09%09%7D%0A%20%20%20%20%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%20%20%20%20%65%6C%73%65%0A%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%09%63%6D%64%20%2B%3D%20%22%6F%70%65%6E%20%5C%22%22%20%2B%20%75%72%6C%20%2B%20%22%5C%22%22%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%74%72%79%0A%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%09%09%09%09%73%79%73%74%65%6D%2E%63%61%6C%6C%53%79%73%74%65%6D%28%63%6D%64%29%3B%0A%09%09%09%7D%0A%09%09%09%63%61%74%63%68%28%65%29%0A%09%09%09%7B%0A%09%09%09%09%61%6C%65%72%74%28%65%29%3B%0A%09%09%09%7D%0A%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%63%65%6E%74%65%72%28%29%3B%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%73%68%6F%77%28%29%3B'));    
    };
}


/**
 Creates an instance of the main class and run it
*/
new AlignLayers().run(this);

